var canvas;
var ctx;
var canvas_w, canvas_h;

var intervalId = 0;  /* stop event loop */

/* ball */
var x, y;
var r;
var fill;  /* RGB transparency */
var dx, dy;

/* paddle */
var  paddle_x;
var  paddle_w, paddle_h;
var  paddle_fill;

/* bricks */
var bricks;
var bricks_r, bricks_c;
var bricks_w, bricks_h;
var bricks_padding;
var bricks_fill;

/* keyboard */
var rightDown;
var leftDown;

/* mouse */
var canvasMinX;
var canvasMaxX;

//set rightDown or leftDown if the right or left keys are down
function onKeyDown(evt) {
    if (evt.keyCode == 39)
        rightDown = true;
    else if (evt.keyCode == 37)
        leftDown = true;
}

//and unset them when the right or left key is released
function onKeyUp(evt) {
    if (evt.keyCode == 39)
        rightDown = false;
    else if (evt.keyCode == 37)
        leftDown = false;
}

function onMouseMove(evt) {
    if (evt.layerX > canvasMinX && evt.layerX < canvasMaxX)
        paddle_x = evt.layerX - canvasMinX - paddle_w/2;
}

function clear_rect(){
    ctx.clearRect(0, 0, canvas_w, canvas_h);
}

function draw_circle(x, y, r, fill) {
    ctx.fillStyle = fill;
    ctx.beginPath();
    ctx.arc(x, y, r, 0, Math.PI*2, true);
    ctx.closePath();
    ctx.fill();
}

function draw_rect(x, y, w, h, fill) {
    ctx.fillStyle = fill;
    ctx.beginPath();
    ctx.rect(x,y,w,h);
    ctx.closePath();
    ctx.fill();
}

function draw_bricks() {
    for (var i=0; i < bricks_r; i++)
      for (var j=0; j < bricks_c; j++)
            if (bricks[i][j] == 1)
                draw_rect((j * (bricks_w + bricks_padding)) + bricks_padding, 
                     (i * (bricks_h + bricks_padding)) + bricks_padding,
                     bricks_w, bricks_h, bricks_fill);
}

function move_paddle(){
    if (rightDown)
        paddle_x += 5;
    else if (leftDown)
        paddle_x -= 5;

    /* keep paddle in box */
    if (paddle_x < 0)
        paddle_x = 0
    if (paddle_x + paddle_w > canvas_w)
        paddle_x = canvas_w - paddle_w;
}

function move_ball(){
    if (x + dx + r > canvas_w || x + dx - r < 0)
        dx = -dx;

    if (y + dy - r < 0)   /* ball at the top */
        dy = -dy;
    else
        if (y + dy + r > canvas_h - paddle_h) {   /* ball at the bottom */
            if (x > paddle_x && x < paddle_x + paddle_w) {
                dx = 8 * ((x-(paddle_x + paddle_w/2))/paddle_w);
                dy = -dy;
            }
            else if (y + dy + r > canvas_h) {
                //game over, so stop the animation
                clearInterval(intervalId);
            }
        }

    x += dx;
    y += dy;
}

function check_wall_standing() {
    var last_brick = true;

    for (var i=0; i < bricks_r; i++)
        for (var j=0; j < bricks_c; j++)
            if (bricks[i][j] == 1){
                last_brick = false;
                break;
            }

    if (last_brick) {
        //game over, so stop the animation
        clearInterval(intervalId);
    }
}

function check_bricks()
{
    //have we hit a brick?
    rowheight = bricks_h + bricks_padding;
    colwidth = bricks_w + bricks_padding;
    row = Math.floor(y/rowheight);
    col = Math.floor(x/colwidth);
    //if so, reverse the ball and mark the brick as broken
    if (y < bricks_r * rowheight && row >= 0 && col >= 0 && bricks[row][col] == 1) {
        dy = -dy;
        bricks[row][col] = 0;
        check_wall_standing();
    }
}

function draw() {
    clear_rect();
    draw_circle(x, y, r, fill);
    draw_rect(paddle_x, canvas_h - paddle_h, paddle_w, paddle_h, paddle_fill);
    move_ball();
    move_paddle();
    check_bricks();
    draw_bricks();
}

function init_canvas() {
    canvas = document.getElementById('canvas');
    ctx = canvas.getContext('2d');
    canvas_h = canvas.height;
    canvas_w = canvas.width;
}

function init_ball() {
    x = canvas_w / 2;
    y = canvas_h / 2;
    r = 10;
    fill = "rgba(0, 0, 0, 1)";  /* RGB opacity */
    dx = 1;
    dy = 4;
}

function init_bricks() {
    bricks_r = 5;
    bricks_c = 5;
    bricks_w = (canvas_w / bricks_c) - 1;
    bricks_h = 15;
    bricks_padding = 1;
    bricks_fill = "rgba(0, 0, 0, 1)";

    bricks = new Array(bricks_r);
    for (var i=0; i < bricks_r; i++) {
        bricks[i] = new Array(bricks_c);
        for (var j=0; j < bricks_c; j++)
            bricks[i][j] = 1;
    }
}

function init_paddle(){
    paddle_x = canvas_w / 2;
    paddle_h = 10;
    paddle_w = 75;
    paddle_fill = "rgba(127, 127, 127, 1)";
}

function init_keyboard() {
    rightDown = false;
    leftDown = false;
}

function init_mouse() {
    canvasMinX = canvas.offsetLeft;
    canvasMaxX = canvasMinX + canvas_w;
}

function init_binds(){
    document.addEventListener("keydown", onKeyDown);
    document.addEventListener("keyup", onKeyUp);
    document.addEventListener("mousemove", onMouseMove);
}

function runCode() {
    init_canvas();
    init_ball();
    init_bricks();
    init_paddle();
    init_keyboard();
    init_mouse();
    init_binds();

    /* start event loop */
    intervalId = setInterval(draw, 10);
}